/*
 * Created on Oct 13, 2003
 */
package isac.bridge;

import isac.util.CalcChanged;
import isac.util.Formalization;
import isac.util.formulae.Assumptions;
import isac.util.formulae.CalcHead;
import isac.util.formulae.CalcHeadCompoundID;
import isac.util.formulae.CalcFormula;
import isac.util.formulae.Position;
import isac.util.interfaces.ICalcElement;
import isac.util.interfaces.ICalcIterator;
import isac.util.tactics.Tactic;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Vector;

/**
 * @author Richard Gradischnegg
 * 
 * WN0422 here seems to be the key for simplifying the whole bridge.* drawbacks
 * of the present implementation: # more complicated than necessary #
 * JUnitTestCases require BridgeMain running # BridgeLogger cannot be made
 * serializable, thus not accessible for writing by JUnitTestCases # ...
 */
public interface IBridgeRMI extends Remote {

	public int startCalculation(Formalization f) throws RemoteException;

	public void modifyCalcHead(CalcHead calcHead) throws RemoteException;

	public int setNextTactic(int calcTreeID, Tactic tactic)
			throws RemoteException;

	public Tactic fetchProposedTactic(int calcTreeID) throws RemoteException;

	public Vector getAppliableTactics(int calcTreeID, int scope, Position pos)
			throws RemoteException;

	public void completeCalcHead(int calcTreeID) throws RemoteException;

	public void completeCalcHead(int calcTreeID, CalcHead calcHead,
			int completeItem) throws RemoteException;

	public CalcChanged autoCalculate(int calcTreeID, int scope, int nSteps)
			throws RemoteException;

	public CalcChanged replaceFormula(int calcTreeID, CalcFormula f)
			throws RemoteException;

	public CalcChanged appendFormula(int id, CalcFormula f)
			throws RemoteException;

	public CalcChanged intermediateSteps(int id, ICalcIterator ci)
			throws RemoteException;

	public Vector getElementsFromTo(int id, ICalcIterator iterator_from,
			ICalcIterator iterator_to, Integer level,
			boolean result_includes_tactics) throws RemoteException;

	public int iterator(int calcTreeID) throws RemoteException;

	public Position moveRoot(int calcTreeID, int iteratorID)
			throws RemoteException;

	//WN041208 late decision for 'interpretSML(moveUp 1 ([1,2,3],"Frm"))'
	//WN041208 or 'interpretSML(moveActiveUp 1)' requires reduncancy here:
	//WN041208 these 2 args are redundant: (1)vvvvvvvvvvv (2)vvvvvvvv
	public Position moveUp(int calcTreeID, int iteratorID, Position p)
			throws RemoteException;

	public Position moveActiveFormula(int calcTreeID, Position p)
			throws RemoteException;

	public Position moveDown(int calcTreeID, int iteratorID, Position p)
			throws RemoteException;

	public Position moveLevelUp(int calcTreeID, int iteratorID, Position p)
			throws RemoteException;

	public Position moveLevelDown(int calcTreeID, int iteratorID, Position p)
			throws RemoteException;

	public Position moveCalcHead(int calcTreeID, int iteratorID, Position p)
			throws RemoteException;

	public boolean moveTactic(int calcTreeID, int iteratorID)
			throws RemoteException;

	public boolean moveFormula(int calcTreeID, int iteratorID)
			throws RemoteException;

	/**
	 * @deprecated in favour of getFormula, getTactic, getAssumption
	 */
	public ICalcElement getElement(int calcTreeID, Position p)
			throws RemoteException;

	public ICalcElement getFormula(int calcTreeID, Position p)
			throws RemoteException;

	public Tactic getTactic(int calcTreeID, Position pos)
			throws RemoteException;

	public Assumptions getAssumptions(int calcTreeID, Position pos)
			throws RemoteException;

	public Assumptions getAccumulatedAssumptions(int calcTreeID, Position pos)
			throws RemoteException;

	public void tryRefine(int calcTreeID, CalcHeadCompoundID problemID)
			throws RemoteException;

	public void tryMatch(int calcTreeID, CalcHeadCompoundID problemID)
			throws RemoteException;

	public boolean destruct(int calcTreeID) throws RemoteException;

	public Vector saveCalcTree(int calcTreeID) throws RemoteException;

	public int loadCalcTree(Vector v) throws RemoteException;

}