/*
 * Created on Jul 15, 2003
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package isac.browserdialog;

import isac.session.OManager;
import isac.wsdialog.DGuide;

import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.xmlrpc.XmlRpcClient;
import org.apache.xmlrpc.XmlRpcException;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * @author gismo
 * 
 * example-implementation for the BrowserDialog
 */
public class BrowserDialog extends UnicastRemoteObject implements BDialog {
  private String id_;

  /**
   * @param session
   *          ID of the session the BrowserDialog is instantiated for - used for
   *          filtering
   */
  public BrowserDialog(String session) throws RemoteException {
    id_ = session;
  }

  private XmlRpcClient client_ = null;

  /*
   * public Vector loadXMLFile(String filename){ Vector parameter = new
   * Vector(); parameter.add(filename); return callKEStore("KEStore.getXMLFile",
   * parameter); }
   */

  /**
   * This method loads binary-files from the KEStore and is accessible through
   * RMI
   * 
   * @param type
   *          Type of KE-Object to load
   * @param filename
   *          id of the binaryfile to load
   * @return binary-file as byte array or empty bytearray if file was not found
   */
  public byte[] loadBinary(String type, String filename) throws RemoteException {
    Vector keresult = loadBinary(id_, type, filename);
    return (byte[]) keresult.elementAt(0);
  }

  /**
   * This method loads binary-files from the KEStore. This method is the XML-RPC
   * version and likely to be dropped in further development.
   * 
   * @param session
   *          for gaining informations if the binary-file may be loaded
   * @param type
   *          Type of KE-Object to load
   * @param filename
   *          id of the binaryfile to load
   * @return binary-file as byte array or empty bytearray if file was not found
   */
  protected Vector loadBinary(String session, String type, String filename) {
    Vector param = new Vector();
    param.add(type);
    param.add(filename);
    return callKEStore("KEStore.loadBinary", param);
  }

  /**
   * Loads the hierarchy of the type <code>type</code> from the KE-Store.
   * Accessible via RMI
   * 
   * @param type
   *          requestet information-type to load the hierarchy for. e.g. pbl,
   *          met, or exp
   * @return the hierarchy in its XML-representation. The result might be
   *         filtered according to the current session the BrowserDialog was
   *         instantiated for
   */
  public String loadHierarchy(String type) throws RemoteException {
    Vector keresult = loadHierarchy(id_, type);
    if (keresult.size() == 0)
      return null;
    return (String) keresult.elementAt(0);
  }

  /**
   * Loads the hierarchy of the type <code>type</code> from the KE-Store.
   * XML-RPC version
   * 
   * @param session
   *          ID of the session to load the KEObject for.
   * @param type
   *          requestet information-type to load the hierarchy for. e.g. pbl,
   *          met, or exp
   * @return the hierarchy in its XML-representation. The result might be
   *         filtered according to the current session the BrowserDialog was
   *         instantiated for
   */
  protected Vector loadHierarchy(String session, String type) {
    Vector param = new Vector();
    param.add(type);
    return callKEStore("KEStore.loadHierarchy", param);
  }

  /**
   * Loads the requested KE-Object of the type <code>type</code> from the
   * KE-Store and evaluates it using the WorksheetDialog identified by
   * <code>wd_id</code>. Accessible via RPC. <code>type</code> and
   * <code>content_id</code> together give the unique id. One and the same
   * <code>content_id</code> might be used in different types (
   * <code>type</code> can be pbl, or met for the basic items of the
   * knowledge-base as well as an identifyer used for different courses). If
   * objects with equal <code>content_id</code> s reside in different types,
   * their only difference should reside within the description.
   * 
   * @param type
   *          requestet information-type to load the content for. e.g. pbl, met,
   *          or exp
   * @param content_id
   *          identifies the KE-Object within the <code>type</code>
   * @param wd_id
   *          identifies the WorksheetDialog to connect for evaluation
   * @return requested KE-Object in its XML-representation. The result might be
   *         filtered according to the current session the BrowserDialog was
   *         instantiated for
   */
  public String evaluateContent(String type, String filename, String wd_id)
      throws RemoteException {
    DGuide dg = getDGuide(wd_id);
    String content = loadContent(type, filename);
    return dg.toString();//TODO put the real method for evaluating an problem
                         // in
  }

  /**
   * Loads the requested KE-Object of the type <code>type</code> from the
   * KE-Store <code>type</code> and <code>content_id</code> together give
   * the unique id. One and the same <code>content_id</code> might be used in
   * different types (<code>type</code> can be pbl, or met for the basic
   * items of the knowledge-base as well as an identifyer used for different
   * courses). If objects with equal <code>content_id</code> s reside in
   * different types, their only difference should reside within the
   * description.
   * 
   * @param type
   *          requestet information-type to load the content for. e.g. pbl, met,
   *          or exp
   * @param content_id
   *          identifies the KE-Object within the <code>type</code>
   * @return requested KE-Object in its XML-representation. The result might be
   *         filtered according to the current session the BrowserDialog was
   *         instantiated for
   */
  public String loadContent(String type, String filename)
      throws RemoteException {
    Vector keresult = loadContent(id_, type, filename);
    return (String) keresult.elementAt(0);
  }

  public String loadContent(String type, Vector idvect) {
    Vector keresult = loadContent(id_, type, idvect);
    return (String) keresult.elementAt(0);
  }

  protected Vector loadContent(String session, String type, Vector idvect) {
    Vector parameter = new Vector();
    parameter.add(type);
    parameter.add(idvect);
    Vector result = callKEStore("KEStore.loadContent", parameter);
    if (result.size() == 0) {
      result.add(getErrorXML("Sorry, proper Content could not be found"));
      return result;
    }
    return filterContent(session, result);
  }

  protected Vector loadContent(String session, String type, String filename) {
    Vector parameter = new Vector();
    parameter.add(type);
    parameter.add(filename);
    Vector result = callKEStore("KEStore.loadContent", parameter);
    if (result.size() == 0) {
      result.add(getErrorXML("Sorry, file " + filename + " not found !"));
      return result;
    }

    return filterContent(session, result);
  }

  /**
   * Loads the requested KE-Object of the type <code>type</code> from the
   * KE-Store. <code>type</code> and <code>content_id</code> together give
   * the unique id. One and the same <code>content_id</code> might be used in
   * different types (<code>type</code> can be pbl, or met for the basic
   * items of the knowledge-base as well as an identifyer used for different
   * courses). If objects with equal <code>content_id</code> s reside in
   * different types, their only difference should reside within the
   * description. XML-RPC version.
   * 
   * @param session
   *          session to fetch the KE-Object for.
   * @param type
   *          requestet information-type to load the content for. e.g. pbl, met,
   *          or exp
   * @param content_id
   *          identifies the KE-Object within the <code>type</code>
   * @return requested KE-Object in its XML-representation. The result might be
   *         filtered according to the session given py parameter
   *         <code>session</code>
   */
  private Vector filterContent(String session, Vector result) {
    if (!session.equals("")) //no filtering necessary
      return result;
    try {
      TransformerFactory tFactory = TransformerFactory.newInstance();
      Transformer transformer = tFactory.newTransformer();

      String content = (String) result.firstElement();
      DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
      DocumentBuilder db = dbf.newDocumentBuilder();
      StringReader sr = new StringReader(content);
      Document document = db.parse(new InputSource(sr));
      DOMSource source = new DOMSource(document);
      DOMResult dom_result = new DOMResult();

      transformer.transform(source, dom_result);
      Source filtered_source = filter(dom_result.getNode());
      StringWriter output = new StringWriter();
      StreamResult filterresult = new StreamResult(output);
      transformer.transform(filtered_source, filterresult);
      String resultxml = output.toString();
      System.out.print(resultxml);

      result = new Vector();
      result.add(resultxml);
    } catch (TransformerConfigurationException e) {
      result.removeAllElements();
      result.add(getErrorXML("TransformerConfigurationEception: "
          + e.getMessage()));
      e.printStackTrace();
    } catch (ParserConfigurationException e) {
      result.removeAllElements();
      result.add(getErrorXML("ParserConfigurationEception: " + e.getMessage()));
      e.printStackTrace();
    } catch (SAXException e) {
      result.removeAllElements();
      result.add(getErrorXML("SAXException: " + e.getMessage()));
      e.printStackTrace();
    } catch (IOException e) {
      result.removeAllElements();
      result.add(getErrorXML("IOEception: " + e.getMessage()));
      e.printStackTrace();
    } catch (TransformerException e) {
      result.removeAllElements();
      result.add(getErrorXML("TransformerEception: " + e.getMessage()));
      e.printStackTrace();
    }

    return result;
  }

  /*
   * @param string @return
   */
  private String getErrorXML(String string) {
    return "<ERROR><DESCRIPTION> <h1> Error in file BrowserDialog.java </h1>"
        + string + "</DESCRIPTION></ERROR>";
  }

  /*
   * @param node
   */
  private Source filter(Node node) {
    recFilter(node, "EVAL_PRECOND");
    recFilter(node, "CAS");
    recFilter(node, "RELATE");
    return new DOMSource(node);
  }

  private void recFilter(Node node, String removename) {
    NodeList nlist = node.getChildNodes();
    System.out.println(nlist);
    for (int indx = 0; indx < nlist.getLength(); indx++) {
      Node childnode = nlist.item(indx);
      System.out.println("removename:" + removename);
      if (childnode == null) {
        System.out.println("child is null !!!");
        return;
      }
      System.out.println("localname:" + childnode.getLocalName());
      if (childnode.getLocalName() == null)
        System.out.println("localname is null !");
      else if (childnode.getLocalName().equals(removename)) {
        System.out.println("i should have been deleted");
        node.removeChild(childnode);
      } else
        recFilter(childnode, removename);
    }
  }

  /**
   * Encapsulates the communication with KEStore. On each call, a new
   * XMLRPC-connection to the KESTore is established
   * 
   * @param function:
   *          the function from the KEStore to call
   * @param parameters:
   *          parameters nedded by the called function
   */
  private Vector callKEStore(String function, Vector parameters) {
    Vector errvector = new Vector();
    try {
      if (client_ == null)
        client_ = new XmlRpcClient("http://localhost:1030");
      return (Vector) client_.execute(function, parameters);
    } catch (MalformedURLException exc) {
      errvector.add("BrowserDialog:MalformedURLException");
      exc.printStackTrace();
    } catch (XmlRpcException exc) {
      errvector.add("BrowserDialog:XmlRpcException");
      exc.printStackTrace();
    } catch (IOException exc) {
      errvector.add("BrowserDialog:IOException");
      exc.printStackTrace();
    }
    return errvector;
  }

  private static DGuide getDGuide(String wd_id) {
    String name = "//localhost/isac-ObjectManager";
    try {
      System.out.println("connect to ObjectManager: " + name);
      Object lookup = Naming.lookup(name);
      return ((OManager) lookup).getWorksheetDialog(wd_id);
    } catch (RemoteException exc) {
      System.err.println("no rmiregistry runing on server " + name
          + " no connection possible --- try later");
      System.err.println("serverCom exception: " + exc.getMessage());
    } catch (NotBoundException exc) {
      System.err.println("servercom: notboundexc: " + exc.getMessage());
      System.err.println("server not found - try to connect to Proxy");
    } catch (java.net.MalformedURLException exc) {
      System.err.println("servercom: MalformedURLException: "
          + exc.getMessage());
      exc.printStackTrace();
    }
    return null;
  }

}
