/****************************************************************** 
 * Copyright (c) 2004, Alois Kirchsteiger, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/

package isac.gui;

import isac.gui.util.SpringUtilities;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.*;

/**
 * @author Alois Kirchsteiger
 */
public class LoginScreen extends JDialog implements ActionListener {

  protected JPanel buttonPanel;

  protected JButton loginButton;

  protected JButton exitButton;

  protected JTextField userNameField;

  protected JPasswordField passwordField;

  protected JLabel passwordLabel;

  protected JLabel userNameLabel;

  /** vector to store the <code>ILoginListener</code> s */
  protected Vector listeners;

  public LoginScreen() {
    this.setLayout();
    this.buildButtons();
    this.buildTextFields();
    this.buildLabels();
    this.addElements();
    this.setDimension();
    listeners = new Vector();
  }

  public void addListener(ILoginListener listener) {
    listeners.add(listener);
  }

  public void removeListener(ILoginListener listener)
                                                     throws NoSuchFieldException {
    if (listeners.contains(listener)) {
      listeners.remove(listener);
    } else {
      throw new NoSuchFieldException(
                                     "you tried to remove a Listener who is not in the vector!");
    }
  }

  public void setLayout() {
    this.getContentPane().setLayout(
                                    new BoxLayout(this.getContentPane(),
                                                  BoxLayout.Y_AXIS));
  }

  public void setDimension() {
    this.setSize(new Dimension(350, 250));
    this.setLocation(this.screenCenter());
    this.setResizable(false);
  }

  public void buildButtons() {
    //login button
    loginButton = new JButton("Login");
    loginButton.addActionListener(this);
    loginButton.setName("Login");
    // exit button
    exitButton = new JButton("Exit");
    exitButton.addActionListener(this);
    exitButton.setName("Exit");
  }

  public void buildTextFields() {
    userNameField = new JTextField("");
    userNameField.setName("UserName");
    //userNameField.setPreferredSize(new Dimension(120,22));
    userNameField.setMaximumSize(new Dimension(250, 22));
    passwordField = new JPasswordField("");
    passwordField.setName("Password");
    passwordField.setEchoChar('*');
    //passwordField.setPreferredSize(new Dimension(120,22));
    passwordField.setMaximumSize(new Dimension(250, 22));
  }

  public void buildLabels() {
    userNameLabel = new JLabel("username: ");
    //userNameLabel.setPreferredSize(new Dimension(100,22));
    userNameLabel.setMaximumSize(new Dimension(150, 22));
    passwordLabel = new JLabel("password: ");
    //passwordLabel.setPreferredSize(new Dimension(100,22));
    passwordLabel.setMaximumSize(new Dimension(150, 22));
  }

  public void addElements() {
    JPanel fieldsPanel = new JPanel();
    JPanel buttonPanel = new JPanel();

    // add username & password labels@fields
    fieldsPanel.setLayout(new SpringLayout());
    fieldsPanel.add(userNameLabel);
    fieldsPanel.add(userNameField);
    fieldsPanel.add(passwordLabel);
    fieldsPanel.add(passwordField);
    SpringUtilities.makeCompactGrid(fieldsPanel, 2, 2, //rows, cols
                                    15, 25, //initialX, initialY
                                    15, 25);//xPad, yPad
    //fieldsPanel.setPreferredSize(new Dimension(250, 100));

    // add login and exit button
    buttonPanel.setLayout(new SpringLayout());
    buttonPanel.add(loginButton);
    buttonPanel.add(exitButton);
    SpringUtilities.makeCompactGrid(buttonPanel, 1, 2, //rows, cols
                                    40, 40, //initialX, initialY
                                    100, 0);//xPad, yPad

    this.getContentPane().add(fieldsPanel);
    this.getContentPane().add(buttonPanel);
  }

  private Point screenCenter() {
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    int midX = screenSize.width / 2 - this.getWidth() / 2;
    int midY = screenSize.height / 2 - this.getHeight() / 2;
    return new Point(midX, midY);
  }

  public void showErrorMsg() {
    JOptionPane.showMessageDialog(this, "could not login, please try again!",
                                  "Login Error!", JOptionPane.ERROR_MESSAGE);
  }

  /*
   * (non-Javadoc)
   * 
   * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
   */
  public void actionPerformed(ActionEvent e) {
    if (e.getSource() == loginButton) {
      fireLoginEvent(new LoginEvent(LoginEvent.LOGIN_TYPE_TRY_TO_LOGIN,
                                    userNameField.getText(),
                                    new String(passwordField.getPassword())));
    } else if (e.getSource() == exitButton) {
      fireLoginEvent(new LoginEvent(LoginEvent.LOGIN_TYPE_EXIT));
    } else {
      System.err.println("no valid Action performed in LoginScreen.java!");
    }

  }

  public void fireLoginEvent(LoginEvent loginEvent) {
    for (int i = 0; i < listeners.size(); i++) {
      ((ILoginListener) listeners.get(i)).loginEventOccured(loginEvent);
    }
  }
}
