/****************************************************************** 
* Copyright (c) 2003, Mario Hochreiter, Student of 
* Software Engineering at FH-Hagenberg, Austria. 
* Project member of the isac team at the Institute for 
* Software-Technologie, Graz University of Technology, Austria. 
* 
* Use is subject to license terms.
******************************************************************/

package isac.gui.browser.problem;

import isac.gui.Worksheet;
import isac.gui.browser.ContentPanel;
import isac.gui.browser.TabPanel;
import isac.gui.util.Hierarchy;
import isac.gui.util.ProblemNode;
import isac.util.formulae.CalcHeadCompoundID;

import java.awt.event.*;
import java.util.ResourceBundle;

import javax.swing.*;
import javax.swing.tree.TreePath;

import org.apache.log4j.Logger;

/**
 * @author Mario Hochreiter
 * @version 0.1
 *
 */
public class ProblemBrowserPanel extends TabPanel implements ActionListener {
  static Logger logger = Logger.getLogger(ProblemBrowserPanel.class.getName());
  protected JPopupMenu popupMenu;
  protected JMenuItem tryRefine;
  protected JMenuItem tryMatch;
  protected ProblemNode selNode;
  protected TreePath selPath;
  
  public ProblemBrowserPanel(JFrame root, Hierarchy hierarchy) {
    super(root, hierarchy);
    this.buildMenu();
    this.initMouseListener();   
  }
  
  protected void buildMenu() {
    popupMenu = new JPopupMenu();
    ResourceBundle rb = ResourceBundle.getBundle("ProblemBrowserPanel", root.getLocale());
    tryRefine = new JMenuItem(rb.getString("refineMenuItem"));
	tryMatch = new JMenuItem(rb.getString("matchMenuItem"));
    tryRefine.addActionListener(this);
    tryMatch.addActionListener(this);
    popupMenu.add(tryRefine);
	  popupMenu.add(tryMatch);
	
  }
  
  protected void initMouseListener() {
    ml = new MouseAdapter() {
      public void mousePressed(MouseEvent e) {
          int selRow = viewTree.getRowForLocation(e.getX(), e.getY());
          selPath = viewTree.getPathForLocation(e.getX(), e.getY());
          if(selRow != -1 && selRow != 0) {
              selNode = (ProblemNode)selPath.getLastPathComponent();
              if ( (e.getClickCount() == 1) && (MouseEvent.BUTTON1 == e.getButton())
              	  && isTryMatchAllowed()) {
                logger.debug("clickCount: 1");
				        CalcHeadCompoundID chID = createCalcHeadCompoundIDFromPath();
                doTryMatch(chID);
                // do something
              } else if(e.getClickCount() == 2)  {
                ContentPanel cp = root.getContentPanel();
                if (selNode.getContentref() != null) {
                  cp.setContent("pbl", selNode.getContentref());                                  
                }
                root.showContentFrame();
              } else if (e.getButton() == MouseEvent.BUTTON3) {
                Worksheet ws = root.getActiveWorksheet();
                logger.debug("MouseButton3");
                if (ws != null && isTryMatchAllowed()) {
                  popupMenu.show(e.getComponent(), e.getX(), e.getY());
                }                
                viewTree.setSelectionPath(selPath);
              }
          }
      }
    };
    viewTree.addMouseListener(ml);
  }
  
  // creates the id for the calcHead from the selected path in the tree
  private CalcHeadCompoundID createCalcHeadCompoundIDFromPath() {
    CalcHeadCompoundID chcmpID = new CalcHeadCompoundID();
    Object[] path = selPath.getPath();
    for (int i = path.length - 1; i > 0; i--) {
      chcmpID.addString(((ProblemNode)path[i]).getId());
    }
    return chcmpID;    
  }
  
  private boolean isTryMatchAllowed() {
//    Worksheet ws = root.getActiveWorksheet();
//    return ws == null ? false : ws.isCallProblemAllowed();
  	//FIXME: remove!!! only a hack to show tryMatch/tryRefine functionality
  	return true;
  }
  
  private void doTryMatch(CalcHeadCompoundID chcmpID) {
    Worksheet ws = root.getActiveWorksheet();
    ws.tryMatch(chcmpID);    
  }
  
  private void doTryRefine(CalcHeadCompoundID chcmpID) {
    Worksheet ws = root.getActiveWorksheet();
    ws.tryRefine(chcmpID);    
  }

  public void actionPerformed(ActionEvent e) {
    CalcHeadCompoundID chcmpID = createCalcHeadCompoundIDFromPath();
    if (e.getSource() == tryRefine) {
		  CalcHeadCompoundID chID = createCalcHeadCompoundIDFromPath();
      doTryRefine(chcmpID);
    } else if (e.getSource() == tryMatch) {
		  CalcHeadCompoundID chID = createCalcHeadCompoundIDFromPath();
    	doTryMatch(chcmpID);
    }
  }
}
