/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/

package isac.gui.calcheadviews;

import isac.gui.util.SpringUtilities;
import isac.util.formulae.CalcHead;

import java.util.Locale;

import javax.swing.*;

import org.apache.log4j.Logger;

/**
 * @author Mario Hochreiter
 * @version 1.0
 *  
 */
public class FullCalcHeadView extends CalcHeadView {
  static Logger logger = Logger.getLogger(FullCalcHeadView.class.getName());

  public FullCalcHeadView(Locale locale, CalcHead ch) {
    super(locale);
    this.init();
    this.fillTextFieldsFromCalcHead(ch);
  }

  private void init() {
    this.buildTextFields();
    this.buildLabels();
    this.layoutComponents();
  }

  protected void layoutComponents() {
    this.setLayout(new SpringLayout());
    this.add(labelGiven);
    this.add(txtGiven);
    this.add(labelWhere);
    this.add(txtWhere);
    this.add(labelFind);
    this.add(txtFind);
    this.add(labelRelate);
    this.add(txtRelate);
    this.add(labelProblem);
    this.add(txtProblem);
    this.add(labelMethod);
    this.add(txtMethod);
    this.add(labelTheory);
    this.add(txtTheory);
    SpringUtilities.makeCompactGrid(this, 7, 2, //rows, cols
                                    5, 5, //initialX, initialY
                                    5, 5);//xPad, yPad
  }

  /** Creates the text input fields for this panel. */
  protected void buildTextFields() {
    txtGiven = new JTextPane();
    txtGiven.setBorder(BorderFactory.createEtchedBorder());
    txtWhere = new JTextPane();
    txtWhere.setEditable(false);
    txtWhere.setBorder(BorderFactory.createEtchedBorder());
    txtFind = new JTextPane();
    txtFind.setBorder(BorderFactory.createEtchedBorder());
    txtTheory = new JTextPane();
    txtTheory.setBorder(BorderFactory.createEtchedBorder());
    txtMethod = new JTextPane();
    txtMethod.setBorder(BorderFactory.createEtchedBorder());
    txtProblem = new JTextPane();
    txtProblem.setBorder(BorderFactory.createEtchedBorder());
    txtRelate = new JTextPane();
    txtRelate.setBorder(BorderFactory.createEtchedBorder());
  }

  /**
   * @param ch
   *          CalcHead from wich the data is read to fill out the text fields
   */
  public void fillCalcHeadFromTextFields(CalcHead ch) {
    //methods from abstract superclass CalcHeadView
    this.insertSpecificationFromTextFieldsInCalcHead(ch);
    this.insertTxtGivenInCalcHead(ch);
    this.insertTxtWhereInCalcHead(ch);
    this.insertTxtFindInCalcHead(ch);
    this.insertTxtRelateInCalcHead(ch);
   
    ch.setCalcHeadStatus(CalcHead.CALCHEAD_CORRECT);
  }

  /**
   * @param ch
   *          CalcHead from wich the data is read to fill out the text fields
   */
  public void fillTextFieldsFromCalcHead(CalcHead ch) {
    logger.info(ch.getGiven() + " " + ch.getFind());

    clearTextFields();
    boolean correct;
    correct = this.checkAndFormatCalcHeadItems(ch.getGiven().getItems()
                                                 .iterator(), txtGiven);
    this.colorizeLabel(correct, labelGiven);
    correct = this.checkAndFormatCalcHeadItems(ch.getFind().getItems()
                                                 .iterator(), txtFind);
    this.colorizeLabel(correct, labelFind);
    correct = this.checkAndFormatCalcHeadItems(ch.getWhere().getItems()
                                                 .iterator(), txtWhere);
    this.colorizeLabel(correct, labelWhere);
    correct = this.checkAndFormatCalcHeadItems(ch.getRelate().getItems()
                                                 .iterator(), txtRelate);
    this.colorizeLabel(correct, labelRelate);

    // the items below have no status -> no colorization needed
    this.checkAndFormatCalcHeadItems(ch.getSpecification().getMethod().getID()
                                       .iterator(), txtMethod);
    this.checkAndFormatCalcHeadItems(ch.getSpecification().getProblem().getID()
                                       .iterator(), txtProblem);
    txtTheory.setText(ch.getSpecification().getTheory().getID());
  }

  /** Creates the labels for this panel. */
  protected void buildLabels() {
    labelGiven = new JLabel(resBundle.getString("given"));
    labelWhere = new JLabel(resBundle.getString("where"));
    labelFind = new JLabel(resBundle.getString("find"));
    labelTheory = new JLabel(resBundle.getString("theory"));
    labelMethod = new JLabel(resBundle.getString("method"));
    labelProblem = new JLabel(resBundle.getString("problem"));
    labelRelate = new JLabel(resBundle.getString("relate"));
  }

}
