/*
 * Created on Sep 23, 2003
 *
 */
package isac.util;

import isac.util.interfaces.*;

import java.io.Serializable;

/**
 * Holds the information about changes in a calculation tree originating from
 * the MathEngine
 * 
 * @author Alan Krempler
 */
public class CalcChangedEvent extends CalcEvent {

	boolean completed_;

	/**
	 * @see isac.util.CalcChangedEvent#getLastUnchangedFormula()
	 */
	ICalcIterator last_unchanged_formula_;

	/**
	 * @see isac.util.CalcChangedEvent#getLastDeletedFormula()
	 */
	ICalcIterator last_deleted_formula_;

	/**
	 * @see isac.util.CalcChangedEvent#getLastGeneratedFormula()
	 */
	ICalcIterator last_generated_formula_;

	/*
	 */

	/**
	 * @param transaction_id
	 *            Identifies the transaction which caused the change in the
	 *            data.
	 * @param success
	 *            true, if the transaction which triggered the change has
	 *            completed successfully. This implies that no further updates
	 *            from this transaction are expected. false otherwise. This is
	 *            to ensure that the user can be notified of the progress of
	 *            lengthy calculations.
	 * 
	 * @param last_unchanged_formula:
	 * @see isac.util.CalcChangedEvent#getLastUnchangedFormula()
	 * 
	 * @param last_deleted_formula:
	 * @see isac.util.CalcChangedEvent#getLastDeletedFormula()
	 * 
	 * @param last_generated_formula:
	 * @see isac.util.CalcChangedEvent#getLastGeneratedFormula()
	 */
	public CalcChangedEvent(int transaction_id, boolean completed,
			ICalcIterator last_unchanged_formula,
			ICalcIterator last_deleted_formula,
			ICalcIterator last_generated_formula) {
		transaction_id_ = transaction_id;
		completed_ = completed;
		last_unchanged_formula_ = last_unchanged_formula;
		last_generated_formula_ = last_generated_formula;
		last_deleted_formula_ = last_deleted_formula;
	}

	/**
	 * @return true, if this is the final change originating from the
	 *         transaction which triggered it. false if the change notifies of
	 *         an intermediate result.
	 */
	public boolean isCompleted() {
		return completed_;
	}

	/**
	 * @deprecated after introduction of 3 iterators below
	 */
	public ICalcIterator getFirstChangedFormula() {
		return last_unchanged_formula_;
	}

	/**
	 * @return the iterator pointing at the last formula where the option for
	 *         this change (a tactic or a subsequently input formula) has been
	 *         applied to. All data up to and including this formula (exclusive)
	 *         are not affected by the update. Allows the user to set the active
	 *         formula somewhere else during calculation for this change.
	 */
	public ICalcIterator getLastUnchangedFormula() {
		return last_unchanged_formula_;
	}

	/**
	 * @return the iterator pointing at the last formula in the successional
	 *         sequence of formulae deleted by this event. Deletion of a
	 *         sequence of formulae is ment to precede the generation of
	 *         formulae
	 */
	public ICalcIterator getLastDeletedFormula() {
		return last_deleted_formula_;
	}

	/**
	 * @return the iterator pointing at the last formula in the successional
	 *         sequence of formulae generated by this event. Deletion of a
	 *         sequence of formulae is ment to precede the generation of
	 *         formulae
	 */
	public ICalcIterator getLastGeneratedFormula() {
		return last_generated_formula_;
	}
}