/*
 * Created on Oct 27, 2003
 */

package isac.util.formulae;

import java.io.Serializable;
import java.util.Iterator;
import java.util.Vector;
import org.apache.log4j.Logger;

/**
 * Position of a passive calculation element (Formula or CalcHead)
 * 
 * @author Richard Gradischnegg
 */

public class Position implements Serializable, Comparable {

	private Vector intList;

	private String kind;

	static Logger logger = Logger.getLogger(Position.class.getName());

	public Position() {
		intList = new Vector();
		kind = "Pbl";
	}

	public String toString() {
		return this.toSMLString();
	}

	public void addInt(String intStr) {
		Integer i = Integer.valueOf(intStr);
		intList.add(i);
	}

	public void setKind(String string) {
		kind = string;
	}

	public String getKind() {
		return kind;
	}

	public Vector getIntList() {
		return intList;
	}

	public String toSMLString() {
		StringBuffer sb = new StringBuffer();
		Iterator it = intList.iterator();
		sb.append("([");
		while (it.hasNext()) {
			sb.append(it.next());
			if (it.hasNext())
				sb.append(",");
		}
		sb.append("],");
		sb.append(kind);
		sb.append(")");
		return sb.toString();
	}

	// WN040909 equivalence classes: {Res} | {Frm, Pbl, Met, Und}
	public static int compareKind(String k1, String k2) {
		if (k1.equals("Res")) {
			if (k2.equals("Res"))
				return 0;
			else
				return 1;
		} else {
			if (k2.equals("Res"))
				return -1;
			else
				return 0;
		}
	}

	public boolean equals(Position other) {
		if (this.intList.size() != other.intList.size())
			return false;
		Iterator it1 = this.intList.iterator();
		Iterator it2 = other.intList.iterator();
		while (it1.hasNext()) {
			if (!it1.next().equals(it2.next()))
				return false;
		}
		// WN040909 equivalence classes: {Res} | {Frm, Pbl, Met, Und}
		if (0 == compareKind(this.kind, other.kind))
			return true;
		else
			return false;
		//RG0404
		//if (!this.kind.equals(other.kind)) return false;
		//return true;
	}

	/*
	 * WN040909 surprising complicated order for isac's tree-structure as impl. in SML
	 * @see java.lang.Comparable#compareTo(java.lang.Object) Position is used as
	 *      key in ...
	 * @see CalcStructure + see 'fun posless' in SML
	 */
	public int compareTo(Object other) {
		if (this.intList.size() == 0) {//([],*)..
			if (((Position) other).intList.size() == 0) {
				return compareKind(this.kind, ((Position) other).kind);//([],*)..([],*)
			} else {
				if (this.kind.equals("Res"))
					return 1;//([],Res) > *
				else
					return -1;//([],Frm|Pbl|Met) < *
			}
		} else {
			if (((Position) other).intList.size() == 0) {
				if (((Position) other).kind.equals("Res"))
					return -1;// * < ([],Res)
				else
					return 1;// * > ([],Frm|Pbl|Met)
			} else {// scan to unequal listElements OR to end at min.
				int l1 = this.intList.size();
				int l2 = ((Position) other).intList.size();
				Iterator itr1 = this.getIntList().iterator();
				Iterator itr2 = ((Position) other).getIntList().iterator();
				Iterator min = itr1;//min driven by itr1
				if (l1 > l2)
					min = itr2;//min driven by itr2
				Integer i1 = (Integer) itr1.next();
				Integer i2 = (Integer) itr2.next();
				while (i1.equals(i2) && min.hasNext()) {//min driven by itr1
														// EXOR itr2
					i1 = (Integer) itr1.next();
					i2 = (Integer) itr2.next();
				}
				//logger.debug("-----------------------------");
				//logger.debug("i1= " + i1 + ", i2= " + i2);
				if (i1.equals(i2)) {
					if (l1 == l2)
						return compareKind(this.kind, ((Position) other).kind);//([1,2],*)..([1,2],*)
					else {
						if (l1 < l2) {
							if (this.kind.equals("Res"))
								return 1;//([1,2],Res) > ([1,2,*],*)
							else
								return -1;//([1,2],Frm|Pbl|Met) < ([1,2,*],*)
						} else {
							if (((Position) other).kind.equals("Res"))
								return -1;//([1,2,*],*) < ([1,2],Res)
							else
								return 1;//([1,2,*],*) > ([1,2],Frm|Pbl|Met)
						}
					}
				} else
					return i1.compareTo(i2);//([#,m],*)<([#,n],*) iff m<n
			}
		}
	}

	//WN040909
	public boolean less(Position p) {
		if (this.compareTo(p) < 0)
			return true;
		else
			return false;
	}
}