/*
 * Created on Nov 24, 2003
 */
package isac.util.parser;

import isac.util.Formalization;
import isac.util.Message;
import isac.util.ResponseWrapper;
import isac.util.Variant;
import isac.util.formulae.CalcHead;
import isac.util.formulae.CalcHeadCompoundID;
import isac.util.formulae.CalcHeadItem;
import isac.util.formulae.CalcHeadItemList;
import isac.util.formulae.CalcHeadSimpleID;
import isac.util.formulae.CalcFormula;
import isac.util.formulae.Position;
import isac.util.formulae.Specification;
import isac.util.tactics.*;
import isac.util.tactics.RewriteSet;

import java.io.IOException;
import java.io.StringReader;

import org.apache.commons.digester.Digester;
import org.xml.sax.EntityResolver;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * @author Andreas Griesmayer
 */
public class FormalizationDigest {

	private Digester digester;

	private String dtdPath;

	class BridgeErrorHandler implements ErrorHandler {
		public void error(SAXParseException e) {
			System.err.println("---xml-error " + e);
		}

		public void fatalError(SAXParseException e) {
			System.err.println("---xml-fatalerror " + e);
		}

		public void warning(SAXParseException e) {
			System.err.println("---xml-warning " + e);
		}
	}

	// auxillary class for feeding the dtd
	class BridgeEntityResolver implements EntityResolver {
		String dtd;

		public BridgeEntityResolver(String dtd) {
			this.dtd = dtd;
		}

		public InputSource resolveEntity(String publicId, String systemId)
				throws SAXException, IOException {
			if ((systemId != null) && systemId.endsWith("isac.dtd")) {
				//System.out.println("------->inside entity resolv");
				return new InputSource(new StringReader(dtd));
				//new StringReader("<!ELEMENT dummy EMPTY>"));
			} else {
				return null;
			}
		}
	}

	/**
	 * Constructor: create a new XMLParserDigest object
	 * 
	 * @param dtdPath
	 *            Path to the DTD-File (Document Type Description)
	 */
	//public FormalizationDigest(String dtdPath) {
	public FormalizationDigest() {
		digester = new Digester();
		//   digester.setErrorHandler(new BridgeErrorHandler());
		/*
		 * this.dtdPath = dtdPath; //dtd-File --> String (faster validating)
		 * StringBuffer sb; try { BufferedReader br = new BufferedReader(new
		 * FileReader(dtdPath)); sb = new StringBuffer(); while (br.ready()) {
		 * sb.append(br.readLine() + "\n"); } // Set dtd for parser validation
		 * digester.setEntityResolver(new BridgeEntityResolver(sb.toString())); }
		 * catch (FileNotFoundException e) { e.printStackTrace(); } catch
		 * (IOException e) { e.printStackTrace(); }
		 */

		// Rules for the digester to create a formalization
		digester.addObjectCreate("EXAMPLE/FORMALIZATION", Formalization.class);
		digester.addObjectCreate("*/FORMALIZATION/VARIANT", Variant.class);
		digester.addCallMethod("*/TERMLIST/MATHML/ISA", "addNewIsaString", 0);
		digester
				.addObjectCreate("*/VARIANT/SPECIFICATION", Specification.class);
		digester.addObjectCreate("*/SPECIFICATION/THEORY",
				CalcHeadSimpleID.class);
		digester.addCallMethod("*/SPECIFICATION/THEORY", "setID", 0);
		digester.addSetNext("*/SPECIFICATION/THEORY", "setTheory");

		digester.addObjectCreate("*/SPECIFICATION/PROBLEM",
				CalcHeadCompoundID.class);
		digester
				.addCallMethod("*/SPECIFICATION/PROBLEM/KEY/ID", "addString", 0);
		digester.addSetNext("*/SPECIFICATION/PROBLEM", "setProblem");

		digester.addObjectCreate("*/SPECIFICATION/METHOD",
				CalcHeadCompoundID.class);
		digester.addCallMethod("*/SPECIFICATION/METHOD/KEY/ID", "addString", 0);
		digester.addSetNext("*/SPECIFICATION/METHOD", "setMethod");

		digester.addSetNext("*/VARIANT/SPECIFICATION", "setSpecification");
		digester.addSetNext("*/FORMALIZATION/VARIANT", "addNewVariant");

		// Rules for the digester to create Objects and fill them from XML-Files
		digester.addObjectCreate("ISAC", ResponseWrapper.class);
		digester.addObjectCreate("*/CALCID", CalcHeadSimpleID.class);
		digester.addCallMethod("*/CALCID", "setID", 0);
		digester.addSetRoot("*/CALCID", "setCalcID");

		// Rules for parsing Position
		digester.addObjectCreate("*/POSITION", Position.class);
		digester.addCallMethod("*/POSITION/INTLIST/INT", "addInt", 0);
		digester.addCallMethod("*/POSITION/POS", "setKind", 0);
		digester.addSetNext("*/POSITION", "setPosition");

		//	Rules for parsing Formula
		digester.addObjectCreate("*/REFFORMULA/FORMULA", CalcFormula.class);
		digester.addCallMethod("*/REFFORMULA/FORMULA/MATHML/ISA", "setText", 0);
		digester.addSetNext("*/REFFORMULA/FORMULA", "setResponse");

		//	Rules for parsing CalcHead
		digester.addObjectCreate("*/CALCHEAD", CalcHead.class);
		digester.addCallMethod("*/CALCHEAD/HEAD", "setHead", 0);
		digester.addCallMethod("*/CALCHEAD/BELONGSTO", "setBelongsTo", 0);
		addItemList(digester, "Given");
		addItemList(digester, "Where");
		addItemList(digester, "Find");
		addItemList(digester, "Relate");
		digester.addObjectCreate("*/CALCHEAD/SPECIFICATION",
				Specification.class);
		digester.addObjectCreate("*/CALCHEAD/SPECIFICATION/THEORYID",
				CalcHeadSimpleID.class);
		digester.addCallMethod("*/CALCHEAD/SPECIFICATION/THEORYID", "setID", 0);
		digester.addSetNext("*/CALCHEAD/SPECIFICATION/THEORYID", "setTheory");
		digester.addObjectCreate("*/CALCHEAD/SPECIFICATION/PROBLEMID",
				CalcHeadCompoundID.class);
		digester.addCallMethod(
				"*/CALCHEAD/SPECIFICATION/PROBLEMID/STRINGLIST/STRING",
				"addString", 0);
		digester.addSetNext("*/CALCHEAD/SPECIFICATION/PROBLEMID", "setProblem");
		digester.addObjectCreate("*/CALCHEAD/SPECIFICATION/METHODID",
				CalcHeadCompoundID.class);
		digester.addCallMethod(
				"*/CALCHEAD/SPECIFICATION/METHODID/STRINGLIST/STRING",
				"addString", 0);
		digester.addSetNext("*/CALCHEAD/SPECIFICATION/METHODID", "setMethod");
		digester.addSetNext("*/CALCHEAD/SPECIFICATION", "setSpecification");

		digester.addSetNext("*/CALCHEAD", "setResponse");

		//	Rules for parsing various Tactic Objects
		//	Rules for parsing SimpleTactic
		digester.addObjectCreate("*/SIMPLETACTIC", SimpleTactic.class);
		digester.addCallMethod("*/SIMPLETACTIC/MATHML/ISA", "setArgument", 0);
		digester.addSetProperties("*/SIMPLETACTIC", "name", "name");
		digester.addSetNext("*/SIMPLETACTIC", "setResponse");

		//	Rules for parsing KeyListTactic
		digester.addObjectCreate("*/STRINGLISTTACTIC", StringListTactic.class);
		digester.addCallMethod("*/STRINGLISTTACTIC/STRINGLIST/STRING",
				"addKey", 0);
		digester.addSetProperties("*/STRINGLISTTACTIC", "name", "name");
		digester.addSetNext("*/STRINGLISTTACTIC", "setResponse");

		//	Rules for parsing RewriteSetTactic
		digester.addObjectCreate("*/REWRITESETTACTIC", RewriteSet.class);
		digester.addCallMethod("*/REWRITESETTACTIC/RULESET", "setRuleSet", 0);
		digester.addSetProperties("*/REWRITESETTACTIC", "name", "name");
		digester.addSetNext("*/REWRITESETTACTIC", "setResponse");

		//	Rules for parsing RewriteSetInstTactic
		digester
				.addObjectCreate("*/REWRITESETINSTTACTIC", RewriteSetInst.class);
		digester.addCallMethod("*/REWRITESETINSTTACTIC/RULESET", "setRuleSet",
				0);
		digester.addCallMethod(
				"*/REWRITESETINSTTACTIC/SUBSTITUTION/PAIR/VARIABLE/MATHML/ISA",
				"setVariable", 0);
		digester.addCallMethod(
				"*/REWRITESETINSTTACTIC/SUBSTITUTION/PAIR/VALUE/MATHML/ISA",
				"setValue", 0);
		digester.addSetProperties("*/REWRITESETINSTTACTIC", "name", "name");
		digester.addSetNext("*/REWRITESETINSTTACTIC", "setResponse");

		// Rules for parsing SubProblemTactic
		digester.addObjectCreate("*/SUBPROBLEMTACTIC", SubProblemTactic.class);
		digester.addCallMethod("*/SUBPROBLEMTACTIC/THEORY", "setTheoryID", 0);
		digester.addCallMethod("*/SUBPROBLEMTACTIC/PROBLEM/STRINGLIST/STRING",
				"addProblemString", 0);
		digester.addSetProperties("*/SUBPROBLEMTACTIC", "name", "name");
		digester.addSetNext("*/SUBPROBLEMTACTIC", "setResponse");

		//	Rules for parsing CalcIterator move message
		digester.addObjectCreate("*/CALCITERATOR", Message.class);
		digester.addCallMethod("*/CALCITERATOR/ERROR", "setError", 0);
		digester.addCallMethod("*/CALCITERATOR/OK", "setOK", 0);
		digester.addSetNext("*/CALCITERATOR", "setResponse");

		//	Rules for parsing CalcIterator constructor message
		digester.addObjectCreate("*/ADDUSER", CalcHeadSimpleID.class);
		digester.addCallMethod("*/ADDUSER/USERID", "setID", 0);
		digester.addSetNext("*/ADDUSER", "setResponse");
	}

	private void addItemList(Digester digester, String modPart) {
		String modPartUp = modPart.toUpperCase();
		// Rules for parsing ItemLists in the model part of a CalcHead: Given,
		// Find, Where, Relate.
		digester.addObjectCreate("*/CALCHEAD/MODEL/" + modPartUp,
				CalcHeadItemList.class);
		digester.addObjectCreate("*/CALCHEAD/MODEL/" + modPartUp + "/ITEM",
				CalcHeadItem.class);
		digester.addSetProperties("*/CALCHEAD/MODEL/" + modPartUp + "/ITEM",
				"status", "itemStatus");
		digester.addCallMethod("*/CALCHEAD/MODEL/" + modPartUp
				+ "/ITEM/MATHML/ISA", "setText", 0);
		digester.addSetNext("*/CALCHEAD/MODEL/" + modPartUp + "/ITEM",
				"addItem");
		digester.addSetNext("*/CALCHEAD/MODEL/" + modPartUp, "set" + modPart);
	}

	/**
	 * Set whether or not using the DTD for validating
	 * 
	 * @param validating
	 *            <UL>
	 *            <LI>true: XML is tested for validity against DTD
	 *            <LI>
	 *            <LI>false: XML is only tested for well-formed-ness
	 *            <LI>
	 *            </UL>
	 */
	public void setValidating(boolean validating) {
		digester.setValidating(validating);
	}

	/**
	 * Parse an XML string to Java Object(s)
	 * 
	 * @param xmlString
	 *            the string containing the XML content
	 * @return ResponseWrapper containing the CalcID and the parsed Object
	 */
	public Formalization parseFormalization(String xmlString) {
		Object o = null;
		//  xmlString =
		//	 "<?xml version=\"1.0\"?>"//<!DOCTYPE ISAC SYSTEM \""
		//+ dtdPath
		//+ "\">"
		//	   +"<ISAC>"
		//	   xmlString
		//	   + "</ISAC>";
		//;
		encodeEntities(xmlString);
		try {
			digester.clear();
			//System.out.println("root === " + digester.getRoot());
			o = digester.parse(new InputSource(new StringReader(xmlString)));
			//System.out.println("--------------->" + xmlString);
		} catch (IOException e) {
			e.printStackTrace();
		} catch (SAXException e) {
			e.printStackTrace();
		}
		Formalization form = (Formalization) o;
		//System.out.println(wrapper.getCalcID().toSMLString());
		return form;
	}

	/**
	 * Parse an XML string to Java Object(s)
	 * 
	 * @param xmlString
	 *            the string containing the XML content
	 * @return ResponseWrapper containing the CalcID and the parsed Object
	 */
	public ResponseWrapper parse(String xmlString) {
		Object o = null;
		xmlString = "<?xml version=\"1.0\"?>"//<!DOCTYPE ISAC SYSTEM \""
				//+ dtdPath
				//+ "\">"
				//	   +"<ISAC>"
				+ xmlString
		//	   + "</ISAC>";
		;
		encodeEntities(xmlString);
		try {
			digester.clear();
			//System.out.println("root === " + digester.getRoot());
			o = digester.parse(new InputSource(new StringReader(xmlString)));
			//System.out.println("--------------->" + xmlString);
		} catch (IOException e) {
			e.printStackTrace();
		} catch (SAXException e) {
			e.printStackTrace();
		}
		ResponseWrapper wrapper = (ResponseWrapper) o;
		//System.out.println(wrapper.getCalcID().toSMLString());
		return wrapper;
	}

	public static void encodeEntities(String xmlString) {
		xmlString = xmlString.replaceAll(" & ", "&amp;");
		xmlString = xmlString.replaceAll(" > ", "&gt;");
		xmlString = xmlString.replaceAll(" < ", "&lt;");
	}

	public static void decodeEntities(String xmlString) {
		xmlString = xmlString.replaceAll("&amp;", " & ");
		xmlString = xmlString.replaceAll("&gt;", " > ");
		xmlString = xmlString.replaceAll("&lt;", " < ");
	}
}